"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const service_1 = require("./service");
const fs = require("fs");
const shelljs = require("shelljs");
const path = require("path");
const crypto = require("crypto");
/**
 * @module botbuilder-config
 */
/*
 * Encrypted properties sourced from Microsoft's MSBot CLI source code.
 *
 */
class BotConfig {
    constructor(options) {
        this.options = options;
        this._encryptedProperties = {
            endpoint: ["appPassword"],
            abs: ["appPassword"],
            luis: ["authoringKey", "subscriptionKey"],
            qna: ["subscriptionKey"],
            dispatch: ["authoringKey", "subscriptionKey"],
            ats: ["storageKey"]
        };
        this._algorithm = "aes192";
        this._base64 = "hex";
        this._ascii = "utf8";
        this.options = this.options || {};
        this.init();
    }
    init() {
        const botFile = this.parseBotFile();
        for (let k in botFile) {
            if (botFile.hasOwnProperty(k)) {
                this[k] = botFile[k];
            }
        }
        return this;
    }
    parseBotFile() {
        let botFile = (this.options.botFilePath !== null) ? this.options.botFilePath : this.getBotFileInDirectory();
        return this.parse(botFile);
    }
    getBotFileInDirectory() {
        let botFile;
        let dir = shelljs.pwd().toString();
        let files = fs.readdirSync(dir);
        files.forEach((f) => {
            if (f.endsWith(".bot")) {
                if (botFile !== null) {
                    throw new Error("Error: Multiple *.bot files in this project directory.");
                }
                botFile = path.join(dir, f);
            }
        });
        if (botFile === null) {
            throw new Error("Error: No *.bot file found in the current working directory.");
        }
        return botFile;
    }
    parse(botFile) {
        //Might need to account for other encodings.
        let f = fs.readFileSync(botFile, "utf-8");
        return JSON.parse(f);
    }
    parseService(type, name) {
        let services = [];
        this.services.forEach((s, idx) => {
            if (s.type === type && (name == null || s.name === name)) {
                services.push(s);
            }
        });
        if (services.length === 0) {
            if (name != null) {
                throw new Error(`Error: No services of type: ${type} and name: ${name} found in your bot file.`);
            }
            throw new Error(`Error: No services of type: ${type} found in your bot file.`);
        }
        return new service_1.Service(services[0]);
    }
    getService(type, name) {
        switch (type.toLowerCase()) {
            case "endpoint":
                return this.parseService(type, name);
            case "abs":
                return this.parseService(type, name);
            case "luis":
                return this.parseService(type, name);
            case "qna":
                return this.parseService(type, name);
            case "dispatch":
                return this.parseService(type, name);
            case "ats":
                return this.parseService(type, name);
            case "blob":
                return this.parseService(type, name);
            default:
                throw new Error(`Error: Invalid Bot Service [${type}] specified.`);
        }
    }
    decryptAll() {
        var self = this;
        self.services.forEach((s, idx) => {
            let encryptedProps = self._encryptedProperties[s.type];
            for (let k in s) {
                if (s.hasOwnProperty(k) && encryptedProps.indexOf(k) != -1 && s[k] != "") {
                    s[k] = self.decrypt(s[k]);
                }
            }
        });
        return self;
    }
    encrypt(value) {
        try {
            const c = crypto.createCipher(this._algorithm, this.options.secret);
            let v = c.update(value, this._ascii, this._base64);
            v += c.final(this._base64);
            return v;
        }
        catch (e) {
            console.log(`Error: Encryption of ${value} failed.`);
        }
    }
    decrypt(value) {
        try {
            const d = crypto.createDecipher(this._algorithm, this.options.secret);
            let prop = d.update(value, this._base64, this._ascii);
            prop += d.final(this._ascii);
            return prop;
        }
        catch (e) {
            console.log(`Error: Decryption for ${value} failed.`);
        }
    }
    Endpoint(name) {
        return this.parseService("endpoint", name);
    }
    AzureBotService(name) {
        return this.parseService("abs", name);
    }
    LUIS(name) {
        return this.parseService("luis", name);
    }
    QnAMaker(name) {
        return this.parseService("qna", name);
    }
    Dispatch(name) {
        return this.parseService("dispatch", name);
    }
    AzureTableStorage(name) {
        return this.parseService("ats", name);
    }
    AzureBlobStorage(name) {
        return this.parseService("blob", name);
    }
}
exports.BotConfig = BotConfig;
//# sourceMappingURL=config.js.map